from orlo import app
from orlo.config import config
from orlo.exceptions import InvalidUsage
from flask import jsonify, request, abort
import arrow
import datetime
from orm import db, DbRelease, DbPackage, DbResults
from orlo.util import list_to_string


@app.errorhandler(InvalidUsage)
def handle_invalid_usage(error):
    response = jsonify(error.to_dict())
    response.status_code = error.status_code
    return response


def _create_release(request):
    """
    Create a DbRelease object from a request
    """

    references = request.json.get('references', [])
    if references and type(references) is list:
        references = list_to_string(references)

    platforms = request.json['platforms']
    if platforms and type(platforms) is list:
        platforms = list_to_string(platforms)

    return DbRelease(
        # Required attributes
        platforms=platforms,
        user=request.json['user'],
        # Not required
        notes=request.json.get('notes'),
        team=request.json.get('team'),
        references=references,
    )


def _create_package(release_id, request):
    """
    Create a package object for a release
    """

    return DbPackage(
        release_id,
        request.json.get('name'),
        request.json.get('version'),
        diff_url=request.json.get('diff_url', None),
        rollback=request.json.get('rollback', False),
    )


def _fetch_release(release_id):
    """
    Fetch a release by ID
    """
    rq = db.session.query(DbRelease).filter(DbRelease.id == release_id)
    dbRelease = rq.first()
    if not dbRelease:
        raise InvalidUsage("Release does not exist")
    return dbRelease


def _fetch_package(release_id, package_id):
    """
    Fetch a package, and validate it is part of the release
    """

    # Fetch release first, as it is a good sanity check
    _fetch_release(release_id)
    pq = db.session.query(DbPackage).filter(DbPackage.id == package_id)
    dbPackage = pq.first()

    if not dbPackage:
        raise InvalidUsage("Package does not exist")
    if str(dbPackage.release_id) != release_id:
        raise InvalidUsage("This package does not belong to this release")

    return dbPackage


def _validate_request_json(request):
    try:
        request.json
    except Exception:
        # This is pretty ugly, but we want something more user friendly than "Bad Request"
        raise InvalidUsage("Could not parse JSON document")

    if not request.json:
        raise InvalidUsage('Missing application/json header', status_code=400)


def _validate_release_input(request):
    _validate_request_json(request)
    if 'platforms' not in request.json:
        raise InvalidUsage('JSON doc missing platforms field', status_code=400)
    return True


def _validate_package_input(request, release_id):
    _validate_request_json(request)

    if not 'name' in request.json or not 'version' in request.json:
        raise InvalidUsage("Missing name / version in request body.")
    app.logger.debug("Package request validated, release_id {}".format(release_id))
    return True


def _validate_package_stop_input(request):
    _validate_request_json(request)
    if 'success' not in request.json:
        raise InvalidUsage("Missing success key in JSON doc")


@app.route('/ping', methods=['GET'])
def ping():
    """
    Simple ping test, takes no parameters

    **Example curl**:

    .. sourcecode:: shell

        curl -X GET 'http://127.0.0.1/ping'
    """
    return 'pong'


@app.route('/releases', methods=['POST'])
def post_releases():
    """
    Create a release - the first step in a deployment

    :<json string user: User that is performing the release
    :<json string team: The development team responsible for this release
    :<json array platforms: List of platforms receiving the release
    :<json array references: List of external references, e.g. Jira ticket
    :>json string id: UUID reference to the created release
    :reqheader Content-Type: Must be application/json
    :status 200: Release was created successfully
    :status 400: Invalid request

    **Example curl**:

    .. sourcecode:: shell

        curl -H "Content-Type: application/json" \\
        -X POST \\
        http://127.0.0.1/releases \\
        -d '{"note": "lorem ipsum", "platforms": ["site1"], "references": ["test-ticket"], "team": "A-Team", "user": "aforbes"}'
    """
    _validate_release_input(request)
    release = _create_release(request)

    app.logger.info(
        'Create release {}, references: {}, platforms: {}'.format(
            release.id, release.notes, release.references, release.platforms)
    )

    release.start()

    db.session.add(release)
    db.session.commit()

    return jsonify(id=release.id)


@app.route('/releases/<release_id>/packages', methods=['POST'])
def post_packages(release_id):
    """
    Add a package to a release

    :param string release_id: UUID of the release to add the package to
    :<json string name: Name of the package
    :<json string version: Version of the package
    :<json boolean rollback: Whether this package deploy is a rollback
    :>json string id: UUID reference to the created package
    :reqheader Content-Type: Must be application/json
    :status 200: Package was added to the release successfully
    :status 400: Invalid request
    **Example curl**:

    .. sourcecode:: shell

        curl -H "Content-Type: application/json" \\
        -X POST \\
        http://127.0.0.1/releases/${RELEASE_ID}/packages \\
        -d '{"name": "test-package", "version": "1.0.1"}'
    """
    _validate_package_input(request, release_id)

    dbRelease = _fetch_release(release_id)
    dbPackage = _create_package(dbRelease.id, request)

    app.logger.info(
        'Create package {}, release {}, name {}, version {}'.format(
            dbPackage.id, dbRelease.id, request.json['name'],
            request.json['version']))

    db.session.add(dbPackage)
    db.session.commit()

    return jsonify(id=dbPackage.id)


@app.route('/releases/<release_id>/packages/<package_id>/results',
           methods=['POST'])
def post_results(release_id, package_id):
    """
    Post the results of a package release

    :param string release_id: Release UUID
    :param string package_id: Package UUID
    :<json string content: Free text field to store what you wish
    :status 204: Package results added successfully
    """
    dbResults = DbResults(package_id, str(request.json))
    app.logger.info("Post results, release {}, package {}".format(
        release_id, package_id))
    db.session.add(dbResults)
    db.session.commit()
    return '', 204


@app.route('/releases/<release_id>/stop', methods=['POST'])
def post_releases_stop(release_id):
    """
    Indicate that a release has finished

    This should be called after all packages have also been "stopped".
    In future it may stop any un-stopped packages.

    :param string release_id: Release UUID

    **Example curl**:

    .. sourcecode:: shell

        curl -H "Content-Type: application/json" \\
        -X POST http://127.0.0.1/releases/${RELEASE_ID}/stop
    """
    dbRelease = _fetch_release(release_id)
    # TODO check that all packages have been finished
    app.logger.info("Release stop, release {}".format(release_id))
    dbRelease.stop()

    db.session.add(dbRelease)
    db.session.commit()
    return '', 204


@app.route('/releases/<release_id>/packages/<package_id>/start',
           methods=['POST'])
def post_packages_start(release_id, package_id):
    """
    Indicate that a package has started deploying

    :param string release_id: Release UUID
    :param string package_id: Package UUID
    :status 204:

    **Example curl**:

    .. sourcecode:: shell

        curl -X POST http://127.0.0.1/releases/${RELEASE_ID}/packages/${PACKAGE_ID}/start
    """
    dbPackage = _fetch_package(release_id, package_id)
    app.logger.info("Package start, release {}, package {}".format(
        release_id, package_id))
    dbPackage.start()

    db.session.add(dbPackage)
    db.session.commit()
    return '', 204


@app.route('/releases/<release_id>/packages/<package_id>/stop',
           methods=['POST'])
def post_packages_stop(release_id, package_id):
    """
    Indicate that a package has finished deploying

    **Example curl**:

    .. sourcecode:: shell

        curl -H "Content-Type: application/json" \\
        -X POST http://127.0.0.1/releases/${RELEASE_ID}/packages/${PACKAGE_ID}/stop \\
        -d '{"success": "true"}'
    """
    _validate_request_json(request)
    success = request.json['success']

    dbPackage = _fetch_package(release_id, package_id)
    app.logger.info("Package stop, release {}, package {}, success {}".format(
        release_id, package_id, success))
    dbPackage.stop(success=success)

    db.session.add(dbPackage)
    db.session.commit()
    return '', 204


@app.route('/releases', methods=['GET'])
@app.route('/releases/<release_id>', methods=['GET'])
def get_releases(release_id=None):
    """
    Return a list of releases to the client, filters optional

    :param string release_id: Optionally specify a single release UUID to fetch. This does not disable filters.
    :query string package_name: Filter releases by package name
    :query string user: Filter releases by user the that performed the release
    :query string platform: Filter releases by platform
    :query string stime_before: Only include releases that started before timestamp given
    :query string stime_after: Only include releases that started after timestamp given
    :query string ftime_before: Only include releases that finished before timestamp given
    :query string ftime_after: Only include releases that finished after timestamp given
    :query int duration_less: Only include releases that took less than (int) seconds
    :query int duration_greater: Only include releases that took more than (int) seconds
    :query string team: Filter releases by team

    **Note for time arguments**:
        The timestamp format you must use is specified in /etc/orlo.conf. All times are UTC.

    .. versionadded:: 0.0.1
    """
    query = db.session.query(DbRelease).order_by(DbRelease.stime.asc())

    if release_id:
        query = query.filter(DbRelease.id == release_id)
    if 'package_name' in request.args:
        query = query.join(DbPackage).filter(DbPackage.name == request.args['package_name'])
    if 'user' in request.args:
        query = query.filter(DbRelease.user == request.args['user'])
    if 'platform' in request.args:
        query = query.filter(
            DbRelease.platforms.like('%{}%'.format(request.args['platform']))
        )
    if 'stime_before' in request.args:
        t = arrow.get(request.args['stime_before'])
        query = query.filter(DbRelease.stime <= t)
    if 'stime_after' in request.args:
        t = arrow.get(request.args['stime_after'])
        query = query.filter(DbRelease.stime >= t)
    if 'ftime_before' in request.args:
        t = arrow.get(request.args['ftime_before'])
        query = query.filter(DbRelease.ftime <= t)
    if 'ftime_after' in request.args:
        t = arrow.get(request.args['ftime_after'])
        query = query.filter(DbRelease.ftime >= t)
    if 'duration_less' in request.args:
        td = datetime.timedelta(seconds=int(request.args['duration_less']))
        query = query.filter(DbRelease.duration < td)
    if 'duration_greater' in request.args:
        td = datetime.timedelta(seconds=int(request.args['duration_greater']))
        query = query.filter(DbRelease.duration > td)
    if 'team' in request.args:
        query = query.filter(DbRelease.team == request.args['team'])

    releases = query.all()
    app.logger.debug("Returning {} releases".format(len(releases)))
    output = []
    for r in releases:
        output.append(r.to_dict())

    return jsonify(releases=output), 200



from orlo import app
from orlo.orm import db
import os
from flask import Flask, abort, request, jsonify, g, url_for
from flask.ext.sqlalchemy import SQLAlchemy
from flask.ext.httpauth import HTTPBasicAuth
from passlib.apps import custom_app_context as pwd_context
from itsdangerous import (TimedJSONWebSignatureSerializer
                          as Serializer, BadSignature, SignatureExpired)

# initialization
app = Flask(__name__)
app.config['SECRET_KEY'] = 'Who lives in a pineapple under the sea?'
app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///orlo.db'
app.config['SQLALCHEMY_COMMIT_ON_TEARDOWN'] = True
app.debug = True

# extensions
db = SQLAlchemy(app)
auth = HTTPBasicAuth()


class User(db.Model):
    __tablename__ = 'users'
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(32), index=True)
    password_hash = db.Column(db.String(64))

    def hash_password(self, password):
        self.password_hash = pwd_context.encrypt(password)

    def verify_password(self, password):
        return pwd_context.verify(password, self.password_hash)

    def generate_auth_token(self, expiration=600):
        s = Serializer(app.config['SECRET_KEY'], expires_in=expiration)
        return s.dumps({'id': self.id})

    @staticmethod
    def verify_auth_token(token):
        s = Serializer(app.config['SECRET_KEY'])
        try:
            data = s.loads(token)
        except SignatureExpired:
            return None  # valid token, but expired
        except BadSignature:
            return None  # invalid token
        user = User.query.get(data['id'])
        return user


@auth.verify_password
def verify_password(username_or_token, password):
    # first try to authenticate by token
    user = User.verify_auth_token(username_or_token)
    if not user:
        # try to authenticate with username/password
        user = User.query.filter_by(username=username_or_token).first()
        if not user or not user.verify_password(password):
            return False
    g.user = user
    return True


@app.route('/auth/users', methods=['POST'])
def new_user():
    username = request.json.get('username')
    password = request.json.get('password')
    if username is None or password is None:
        abort(400)  # missing arguments
    if User.query.filter_by(username=username).first() is not None:
        abort(400)  # existing user
    user = User(username=username)
    user.hash_password(password)
    db.session.add(user)
    db.session.commit()
    return (jsonify({'username': user.username}), 201,
            {'Location': url_for('get_user', id=user.id, _external=True)})


@app.route('/auth/users/<int:id>')
def get_user(id):
    user = User.query.get(id)
    if not user:
        abort(400)
    return jsonify({'username': user.username})


@app.route('/auth/token')
@auth.login_required
def get_auth_token():
    token = g.user.generate_auth_token(600)
    return jsonify({'token': token.decode('ascii'), 'duration': 600})


@app.route('/auth/resource')
@auth.login_required
def get_resource():
    return jsonify({'data': 'Hello, %s!' % g.user.username})


if __name__ == '__main__':
    if not os.path.exists('db.sqlite'):
        db.create_all()
    app.run(debug=True)



from flask import Flask
from logging.handlers import RotatingFileHandler

from orlo.config import config

app = Flask(__name__)

app.config['SQLALCHEMY_DATABASE_URI'] = config.get('db', 'uri')

if config.getboolean('main', 'propagate_exceptions'):
    app.config['PROPAGATE_EXCEPTIONS'] = True

if config.getboolean('logging', 'debug'):
    app.debug = True
app.logger.debug('Debug enabled')

logfile = config.get('logging', 'file')
if logfile != 'disabled':
    handler = RotatingFileHandler(
        logfile,
        maxBytes=10000,
        backupCount=1,
    )
    app.logger.addHandler(handler)

# Must be imported last
import orlo.views
import orlo.route_import



from __future__ import print_function
from tests.test_contract import OrloTest
from random import randrange
from tests.test_contract import db
from orlo.orm import DbRelease, DbPackage, DbResults
import arrow
import datetime
import uuid

__author__ = 'alforbes'

'''
Tests for the database / orm layer
'''


class OrloDbTest(OrloTest):
    def setUp(self):
        super(OrloDbTest, self).setUp()

        # Add 10 releases
        for _ in range(10):
            self._add_release()

    def _add_release(self):
        r = DbRelease(
            platforms=['PLATFORM1', 'PLATFORM2'],
            user='TEST USER',
            notes='Random note {}'.format(randrange(0, 1000)),
            references=['REF-{}'.format(randrange(99, 1000)) * 2],
            team='TEST TEAM',
        )
        db.session.add(r)

        r.start()
        for _ in range(0, 2):
            self._add_package(r.id)
        r.stop()
        db.session.commit()

    def _add_package(self, release_id):
        """
        Add a package to a release
        """
        p = DbPackage(
            release_id=release_id,
            name='TestPackage-{}'.format(randrange(0, 10)),
            version=str(randrange(0, 1000)),
        )
        db.session.add(p)
        p.start()
        p.stop(success=True)

    def test_release_types(self):
        """
        Test the types returned by DbRelease objects are OK
        """
        r = db.session.query(DbRelease).first()
        self.assertIs(type(r.id), uuid.UUID)
        self.assertIs(type(r.notes), unicode)
        self.assertIs(type(r.platforms), unicode)
        self.assertIs(type(r.references), unicode)
        self.assertIs(type(list(r.references)), list)
        self.assertIs(type(r.stime), arrow.arrow.Arrow)
        self.assertIs(type(r.ftime), arrow.arrow.Arrow)
        self.assertIs(type(r.duration), datetime.timedelta)
        self.assertIs(type(r.team), unicode)

    def test_package_types(self):
        """
        Test the types returned by DbPackage objects are OK
        """
        p = db.session.query(DbPackage).first()
        self.assertIs(type(p.id), uuid.UUID)
        self.assertIs(type(p.name), unicode)
        self.assertIs(type(p.stime), arrow.arrow.Arrow)
        self.assertIs(type(p.ftime), arrow.arrow.Arrow)
        self.assertIs(type(p.duration), datetime.timedelta)
        self.assertIs(type(p.status), unicode)
        self.assertIs(type(p.version), unicode)



from __future__ import print_function, unicode_literals
import json

__author__ = 'alforbes'


def string_to_list(string):
    """
    Load a list from a string

    :param string:
    :return:
    """
    if string is None:
        return []

    if '[' in string and \
            ']' in string and \
            ('"' in string or "'" in string):
        # Valid list syntax, presumably
        return json.loads(string.replace("'", '"'))
    else:
        # assume just one item
        return [string]


def list_to_string(array):
    """
    Convert a list to a string for storage in the DB

    :param array:
    :return:
    """
    if not array:
        return []

    return '["' + '", "'.join(array) + '"]'



#!/usr/bin/env python

# from distutils.core import setup
from setuptools import setup, find_packages
import multiprocessing  # nopep8

setup(
    name='orlo',
    version='0.0.2',
    description='Deployment data capture API',
    author='Alex Forbes',
    author_email='alforbes@ebay.com',
    license='GPL',
    long_description=open('README.md').read(),
    url='https://github.com/eBayClassifiedsGroup/orlo',
    packages=[
        'orlo',
    ],
    include_package_data=True,
    install_requires=[
        'Flask',
        'Flask-Migrate',
        'Flask-SQLAlchemy',
        'arrow',
        'gunicorn',
        'psycopg2',
        'pytz',
        'sphinxcontrib-httpdomain',
        'sqlalchemy-utils',
    ],
    tests_require=[
        'Flask-Testing',
    ],
    test_suite='tests',
)



from __future__ import print_function, unicode_literals
from datetime import datetime, timedelta
import orlo
import json
import uuid
from flask.ext.testing import TestCase
from orlo.orm import db, DbPackage, DbRelease
from orlo.config import config


class OrloTest(TestCase):
    """
    Base class for tests which contains the methods required to move
    releases and packages through the workflow
    """

    def create_app(self):
        app = orlo.app
        app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite://'
        app.config['TESTING'] = True
        app.config['DEBUG'] = True
        app.config['TRAP_HTTP_EXCEPTIONS'] = True
        app.config['PRESERVE_CONTEXT_ON_EXCEPTION'] = False

        return orlo.app

    def setUp(self):
        db.create_all()

    def tearDown(self):
        db.session.remove()
        db.drop_all()

    def _create_release(self,
                        user='testuser',
                        team='test team',
                        platforms='test_platform',
                        references='TestTicket-123',
                        ):
        """
        Create a release using the REST API

        :param user:
        :param team:
        :param platforms:
        :param reference:
        :return: the release ID
        """

        response = self.client.post(
            '/releases',
            data=json.dumps({
                'note': 'test note lorem ipsum',
                'platforms': platforms,
                'references': references,
                'team': team,
                'user': user,
            }),
            content_type='application/json',
        )
        self.assert200(response)
        return response.json['id']

    def _create_package(self, release_id,
                        name='test-package',
                        version='1.2.3',
                        diff_url=None,
                        rollback=False,
                        ):
        """
        Create a package using the REST API

        :param release_id: release id to create the package for
        :param name:
        :param version:
        :param diff_url:
        :param rollback:
        :return: package id
        """
        doc = {
            'name': name,
            'version': version,
        }

        if diff_url:
            doc['diff_url'] = diff_url
        if rollback:
            doc['rollback'] = rollback

        response = self.client.post(
            '/releases/{}/packages'.format(release_id),
            data=json.dumps(doc),
            content_type='application/json',
        )
        self.assert200(response)
        return response.json['id']

    def _start_package(self, release_id, package_id):
        """
        Start a package using the REST API

        :param release_id:
        :return:
        """

        response = self.client.post(
            '/releases/{}/packages/{}/start'.format(release_id, package_id),
            content_type='application/json',
        )
        self.assertEqual(response.status_code, 204)
        return response

    def _stop_package(self, release_id, package_id,
                      success=True):
        """
        Start a package using the REST API

        :param release_id:
        :return:
        """

        response = self.client.post(
            '/releases/{}/packages/{}/stop'.format(release_id, package_id),
            data=json.dumps({
                'success': str(success),
                'foo': 'bar',
            }),
            content_type='application/json',
        )
        self.assertEqual(response.status_code, 204)
        return response

    def _stop_release(self, release_id):
        """
        Stop a release using the REST API

        :param release_id:
        :return:
        """
        response = self.client.post(
            '/releases/{}/stop'.format(release_id),
            content_type='application/json',
        )

        self.assertEqual(response.status_code, 204)
        return response

    def _create_finished_release(self):
        """
        Create a completed release, with default values, that has gone through
        the whole workflow
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)

        self._start_package(release_id, package_id)
        self._stop_package(release_id, package_id)
        self._stop_release(release_id)

        return release_id


class PostContractTest(OrloTest):
    """
    Test the HTTP POST contract
    """

    def test_create_release(self):
        """
        Create a release
        """
        release_id = self._create_release()
        self.assertEqual(uuid.UUID(release_id).get_version(), 4)

    def test_create_package(self):
        """
        Create a package
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)
        self.assertEqual(uuid.UUID(package_id).get_version(), 4)

    def test_add_results(self):
        """
        Add the results of a package deploy
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)

        results_response = self.client.post(
            '/releases/{}/packages/{}/results'.format(
                release_id, package_id),
            data=json.dumps({
                'success': 'true',
                'foo': 'bar',
            }),
            content_type='application/json',
        )
        self.assertEqual(results_response.status_code, 204)

    def test_package_start(self):
        """
        Test starting a package
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)

        return release_id, package_id

    def test_package_stop(self):
        """
        Test stopping a package
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)
        self._start_package(release_id, package_id)
        self._stop_package(release_id, package_id)

    def test_release_stop(self):
        """
        Test stopping a release

        As before, calls all the others as we need a full workflow present
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id)
        self._start_package(release_id, package_id)
        self._stop_package(release_id, package_id)
        self._stop_release(release_id)

    def test_create_release_minimal(self):
        """
        Create a release, omitting all optional parameters
        """
        response = self.client.post('/releases',
            data=json.dumps({
                'platforms': ['test_platform'],
                'user': 'testuser',
            }),
            content_type='application/json',
        )
        self.assert200(response)

    def test_diffurl_present(self):
        """
        Test that the diff_url parameter is stored
        """
        test_url = 'http://example.com'

        release_id = self._create_release()
        package_id = self._create_package(release_id, diff_url=test_url)

        q = db.session.query(DbPackage).filter(DbPackage.id == package_id)
        pkg = q.first()
        self.assertEqual(pkg.diff_url, test_url)

    def test_rollback_present(self):
        """
        Test that that rollback parameter is stored
        """
        release_id = self._create_release()
        package_id = self._create_package(release_id, rollback=True)

        q = db.session.query(DbPackage).filter(DbPackage.id == package_id)
        pkg = q.first()

        self.assertEqual(pkg.rollback, True)


class GetContractTest(OrloTest):
    """
    Test the HTTP GET contract
    """

    def _get_releases(self, release_id=None, filters=None):
        """
        Perform a GET to /releases with optional filters
        """

        if release_id:
            path = '/releases/{}'.format(release_id)
        elif filters:
            path = '/releases?{}'.format('&'.join(filters))
        else:
            path = '/releases'

        print("GET {}".format(path))
        results_response = self.client.get(
            path, content_type='application/json',
        )
        self.assertEqual(results_response.status_code, 200)
        r_json = json.loads(results_response.data)
        return r_json

    def test_ping(self):
        """
        Start with something simple
        """
        response = self.client.get('/ping')
        self.assert200(response)
        self.assertEqual(response.data, 'pong')

    def test_get_single_release(self):
        """
        Fetch a single release
        """
        release_id = self._create_release()
        r = self._get_releases(release_id=release_id)

        self.assertEqual(1, len(r['releases']))
        self.assertEqual(release_id, r['releases'][0]['id'])

    def test_get_releases(self):
        """
        Test the list of releases
        """
        for _ in range(0, 3):
            self._create_finished_release()
        results = self._get_releases()
        self.assertEqual(len(results['releases']), 3)

    def test_get_release_filter_package(self):
        """
        Filter on releases that contain a package
        """
        for _ in range(0, 3):
            self._create_finished_release()

        release_id = self._create_release()
        package_id = self._create_package(release_id, name='specific-package')
        results = self._get_releases(filters=[
            'package_name=specific-package'
            ])

        for r in results['releases']:
            for p in r['packages']:
                self.assertEqual(p['name'], 'specific-package')
                self.assertEqual(p['id'], package_id)

        self.assertEqual(len(results['releases']), 1)

    def test_get_release_filter_user(self):
        """
        Filter on releases that were performed by a user
        """
        for _ in range(0, 3):
            self._create_release(user='firstUser')

        for _ in range(0, 2):
            self._create_release(user='secondUser')

        first_results = self._get_releases(filters=['user=firstUser'])
        second_results = self._get_releases(filters=['user=secondUser'])

        self.assertEqual(len(first_results['releases']), 3)
        self.assertEqual(len(second_results['releases']), 2)

        for r in first_results['releases']:
            self.assertEqual(r['user'], 'firstUser')
        for r in second_results['releases']:
            self.assertEqual(r['user'], 'secondUser')

    def test_get_release_filter_platform(self):
        """
        Filter on releases that were on a particular platform
        """
        for _ in range(0, 3):
            self._create_release(platforms='firstPlatform')

        for _ in range(0, 2):
            self._create_release(platforms=['firstPlatform', 'secondPlatform'])

        first_results = self._get_releases(filters=['platform=firstPlatform'])
        second_results = self._get_releases(filters=['platform=secondPlatform'])

        # Should be all releases
        self.assertEqual(len(first_results['releases']), 5)
        # Just those that contain "secondPlatform"
        self.assertEqual(len(second_results['releases']), 2)

        for r in first_results['releases']:
            self.assertIn('firstPlatform', r['platforms'])
        for r in second_results['releases']:
            self.assertIn('secondPlatform', r['platforms'])

    def _get_releases_time_filter(self, field, finished=False):
        """
        Return releases given the time-based filter field

        Abstracts a bit of common code in the stime/ftime tests
        """
        for _ in range(0, 3):
            if finished:
                self._create_finished_release()
            else:
                self._create_release()

        t_format = config.get('main', 'time_format')
        now = datetime.utcnow()

        yesterday = (now - timedelta(days=1)).strftime(t_format)
        tomorrow = (now + timedelta(days=1)).strftime(t_format)

        r_yesterday = self._get_releases(filters=['{}={}'.format(field, yesterday)])
        r_tomorrow = self._get_releases(filters=['{}={}'.format(field, tomorrow)])

        return r_yesterday, r_tomorrow

    def test_get_release_filter_stime_before(self):
        """
        Filter on releases that started before a particular time
        """
        r_yesterday, r_tomorrow = self._get_releases_time_filter('stime_before')

        self.assertEqual(3, len(r_tomorrow['releases']))
        self.assertEqual(0, len(r_yesterday['releases']))

    def test_get_release_filter_stime_after(self):
        """
        Filter on releases that started after a particular time
        """
        r_yesterday, r_tomorrow = self._get_releases_time_filter('stime_after')

        self.assertEqual(0, len(r_tomorrow['releases']))
        self.assertEqual(3, len(r_yesterday['releases']))

    def test_get_release_filter_ftime_before(self):
        """
        Filter on releases that finished before a particular time
        """
        r_yesterday, r_tomorrow = self._get_releases_time_filter(
            'ftime_before', finished=True)

        self.assertEqual(3, len(r_tomorrow['releases']))
        self.assertEqual(0, len(r_yesterday['releases']))

    def test_get_release_filter_ftime_after(self):
        """
        Filter on releases that finished after a particular time
        """
        r_yesterday, r_tomorrow = self._get_releases_time_filter(
            'ftime_after', finished=True)

        self.assertEqual(0, len(r_tomorrow['releases']))
        self.assertEqual(3, len(r_yesterday['releases']))

    def test_get_release_filter_duration_less(self):
        """
        Filter on releases that took less than x seconds

        All releases take a very small amount of time in our tests,
        but all should be <10 and >0 at least!
        """
        for _ in range(0, 3):
            self._create_finished_release()

        r = self._get_releases(filters=['duration_less=10'])
        self.assertEqual(3, len(r['releases']))

        r = self._get_releases(filters=['duration_less=0'])
        self.assertEqual(0, len(r['releases']))

    def test_get_release_filter_duration_greater(self):
        """
        Filter on releases that took greater than x seconds
        """
        for _ in range(0, 3):
            self._create_finished_release()

        r = self._get_releases(filters=['duration_greater=10'])
        self.assertEqual(0, len(r['releases']))

        r = self._get_releases(filters=['duration_greater=0'])
        self.assertEqual(3, len(r['releases']))

    def test_get_release_filter_team(self):
        """
        Filter on releases that a team was responsible for
        """
        for _ in range(0, 3):
            self._create_release(team='firstTeam')

        for _ in range(0, 2):
            self._create_release(team='secondTeam')

        first_results = self._get_releases(filters=['team=firstTeam'])
        second_results = self._get_releases(filters=['team=secondTeam'])

        self.assertEqual(len(first_results['releases']), 3)
        self.assertEqual(len(second_results['releases']), 2)

        for r in first_results['releases']:
            self.assertEqual(r['team'], 'firstTeam')
        for r in second_results['releases']:
            self.assertEqual(r['team'], 'secondTeam')



from __future__ import print_function
import ConfigParser
__author__ = 'alforbes'

config = ConfigParser.ConfigParser()

config.add_section('main')
config.set('main', 'propagate_exceptions', 'true')
config.set('main', 'time_format', '%Y-%m-%dT%H:%M:%SZ')
config.set('main', 'time_zone', 'UTC')

config.add_section('db')
config.set('db', 'uri', 'sqlite://')

config.add_section('logging')
config.set('logging', 'debug', 'false')
config.set('logging', 'file', 'disabled')

config.read('/etc/orlo.conf')



from __future__ import print_function
import arrow
from flask import jsonify, request
from orlo import app
from orlo.orm import db, DbPackage, DbRelease, DbResults
from orlo.views import _validate_request_json
from orlo.util import list_to_string

__author__ = 'alforbes'


@app.route('/import', methods=['GET'])
@app.route('/import/release', methods=['GET'])
def get_import():
    """
    Display a useful message on how to import

    :status 200:
    """

    msg = 'This is the orlo import url'
    return jsonify(message=msg), 200


@app.route('/import/release', methods=['POST'])
def post_import():
    """
    Import a release

    The document must describe a full release, example:
    {
        'platforms': ['test_platform'],
        'ftime': '2015-11-18T19:21:12Z',
        'stime': '2015-11-18T19:21:12Z',
        'team': 'test team',
        'references': ['TestTicket-123'],
        'packages': [
            {
                'status': 'SUCCESSFUL',
                'name': 'test-package',
                'version': '1.2.3',
                'ftime': '2015-11-18T19:21:12Z',
                'stime': '2015-11-18T19:21:12Z',
                'diff_url': None,
            }
        ],
        'user': 'testuser'
    }

    :status 200: The release was accepted
    """

    _validate_request_json(request)

    release = DbRelease(
        platforms=request.json.get('platforms'),
        user=request.json.get('user'),
        notes=request.json.get('notes', None),
        team=request.json.get('team', None),
        references=request.json.get('references'),
    )

    release.ftime = arrow.get(request.json.get('ftime'))
    release.stime = arrow.get(request.json.get('stime'))
    release.duration = release.ftime - release.stime

    db.session.add(release)

    for p in request.json.get('packages'):
        package = DbPackage(
            release_id=release.id,
            name=p['name'],
            version=p['version'],
        )
        package.stime = arrow.get(p['stime'])
        package.ftime = arrow.get(p['ftime'])
        package.duration = package.ftime - package.stime
        package.status = p['status']
        package.diff_url = p['diff_url']

        db.session.add(package)

    db.session.commit()

    return jsonify(release_id=release.id), 200


